package com.ruoyi.system.service.impl;

import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.core.exception.ServiceException;
import java.util.Collection;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.system.api.domain.SysRole;
import java.util.Iterator;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.function.Function;
import java.util.ArrayList;
import com.ruoyi.common.core.utils.SpringUtils;
import com.ruoyi.system.domain.vo.TreeSelect;
import com.ruoyi.common.datascope.annotation.DataScope;
import java.util.List;
import com.ruoyi.system.api.domain.SysDept;
import com.ruoyi.system.mapper.SysRoleMapper;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.mapper.SysDeptMapper;
import org.springframework.stereotype.Service;
import com.ruoyi.system.service.ISysDeptService;

@Service
public class SysDeptServiceImpl implements ISysDeptService
{
    @Autowired
    private SysDeptMapper deptMapper;
    @Autowired
    private SysRoleMapper roleMapper;
    
    @DataScope(deptAlias = "d")
    public List<SysDept> selectDeptList(final SysDept dept) {
        return this.deptMapper.selectDeptList(dept);
    }
    
    public List<TreeSelect> selectDeptTreeList(final SysDept dept) {
        final List<SysDept> depts = ((SysDeptServiceImpl)SpringUtils.getAopProxy((Object)this)).selectDeptList(dept);
        return this.buildDeptTreeSelect(depts);
    }
    
    public List<SysDept> buildDeptTree(final List<SysDept> depts) {
        List<SysDept> returnList = new ArrayList<SysDept>();
        final List<Long> tempList = depts.stream().map((Function<? super Object, ?>)SysDept::getDeptId).collect((Collector<? super Object, ?, List<Long>>)Collectors.toList());
        for (final SysDept dept : depts) {
            if (!tempList.contains(dept.getParentId())) {
                this.recursionFn(depts, dept);
                returnList.add(dept);
            }
        }
        if (returnList.isEmpty()) {
            returnList = depts;
        }
        return returnList;
    }
    
    public List<TreeSelect> buildDeptTreeSelect(final List<SysDept> depts) {
        final List<SysDept> deptTrees = this.buildDeptTree(depts);
        return deptTrees.stream().map((Function<? super Object, ?>)TreeSelect::new).collect((Collector<? super Object, ?, List<TreeSelect>>)Collectors.toList());
    }
    
    public List<Long> selectDeptListByRoleId(final Long roleId) {
        final SysRole role = this.roleMapper.selectRoleById(roleId);
        return this.deptMapper.selectDeptListByRoleId(roleId, role.isDeptCheckStrictly());
    }
    
    public SysDept selectDeptById(final Long deptId) {
        return this.deptMapper.selectDeptById(deptId);
    }
    
    public int selectNormalChildrenDeptById(final Long deptId) {
        return this.deptMapper.selectNormalChildrenDeptById(deptId);
    }
    
    public boolean hasChildByDeptId(final Long deptId) {
        final int result = this.deptMapper.hasChildByDeptId(deptId);
        return result > 0;
    }
    
    public boolean checkDeptExistUser(final Long deptId) {
        final int result = this.deptMapper.checkDeptExistUser(deptId);
        return result > 0;
    }
    
    public boolean checkDeptNameUnique(final SysDept dept) {
        final Long deptId = StringUtils.isNull((Object)dept.getDeptId()) ? -1L : dept.getDeptId();
        final SysDept info = this.deptMapper.checkDeptNameUnique(dept.getDeptName(), dept.getParentId());
        return !StringUtils.isNotNull((Object)info) || info.getDeptId() == (long)deptId;
    }
    
    public void checkDeptDataScope(final Long deptId) {
        if (!SysUser.isAdmin(SecurityUtils.getUserId()) && StringUtils.isNotNull((Object)deptId)) {
            final SysDept dept = new SysDept();
            dept.setDeptId(deptId);
            final List<SysDept> depts = ((SysDeptServiceImpl)SpringUtils.getAopProxy((Object)this)).selectDeptList(dept);
            if (StringUtils.isEmpty((Collection)depts)) {
                throw new ServiceException("没有权限访问部门数据！");
            }
        }
    }
    
    public int insertDept(final SysDept dept) {
        final SysDept info = this.deptMapper.selectDeptById(dept.getParentId());
        if (!"0".equals(info.getStatus())) {
            throw new ServiceException("部门停用，不允许新增");
        }
        dept.setAncestors(info.getAncestors() + "," + dept.getParentId());
        return this.deptMapper.insertDept(dept);
    }
    
    public int updateDept(final SysDept dept) {
        final SysDept newParentDept = this.deptMapper.selectDeptById(dept.getParentId());
        final SysDept oldDept = this.deptMapper.selectDeptById(dept.getDeptId());
        if (StringUtils.isNotNull((Object)newParentDept) && StringUtils.isNotNull((Object)oldDept)) {
            final String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
            final String oldAncestors = oldDept.getAncestors();
            dept.setAncestors(newAncestors);
            this.updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
        }
        final int result = this.deptMapper.updateDept(dept);
        if ("0".equals(dept.getStatus()) && StringUtils.isNotEmpty(dept.getAncestors()) && !StringUtils.equals((CharSequence)"0", (CharSequence)dept.getAncestors())) {
            this.updateParentDeptStatusNormal(dept);
        }
        return result;
    }
    
    private void updateParentDeptStatusNormal(final SysDept dept) {
        final String ancestors = dept.getAncestors();
        final Long[] deptIds = Convert.toLongArray(ancestors);
        this.deptMapper.updateDeptStatusNormal(deptIds);
    }
    
    public void updateDeptChildren(final Long deptId, final String newAncestors, final String oldAncestors) {
        final List<SysDept> children = this.deptMapper.selectChildrenDeptById(deptId);
        for (final SysDept child : children) {
            child.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
        }
        if (children.size() > 0) {
            this.deptMapper.updateDeptChildren((List)children);
        }
    }
    
    public int deleteDeptById(final Long deptId) {
        return this.deptMapper.deleteDeptById(deptId);
    }
    
    private void recursionFn(final List<SysDept> list, final SysDept t) {
        final List<SysDept> childList = this.getChildList(list, t);
        t.setChildren((List)childList);
        for (final SysDept tChild : childList) {
            if (this.hasChild(list, tChild)) {
                this.recursionFn(list, tChild);
            }
        }
    }
    
    private List<SysDept> getChildList(final List<SysDept> list, final SysDept t) {
        final List<SysDept> tlist = new ArrayList<SysDept>();
        for (final SysDept n : list) {
            if (StringUtils.isNotNull((Object)n.getParentId()) && n.getParentId() == (long)t.getDeptId()) {
                tlist.add(n);
            }
        }
        return tlist;
    }
    
    private boolean hasChild(final List<SysDept> list, final SysDept t) {
        return this.getChildList(list, t).size() > 0;
    }
}
